__version__ = '0.8a3'

import unittest
import xml.dom

import cssutils.css


class CSSStyleSheetTestCase(unittest.TestCase):

    def setUp(self):
        self.r1s = '.r1 {\n    color: red;\n    }' # rule string

        self.r1 = cssutils.css.CSSStyleRule() # Rule object
        self.r1.selectorText = '.r1'
        d = cssutils.css.CSSStyleDeclaration()
        d.setProperty('color', 'red')
        self.r1.setStyleDeclaration(d)
    
    def test_init(self):
        css = cssutils.css.CSSStyleSheet()
        self.assertEqual(False, css._readonly)
        self.assertEqual([], css.cssRules)

    def test_readonly(self):
        css = cssutils.css.CSSStyleSheet(readonly=True)
        self.assertEqual(True, css._readonly) # internal...
        self.assertRaises(xml.dom.NoModificationAllowedErr,
                          css.addRule, self.r1)
        self.assertRaises(xml.dom.NoModificationAllowedErr,
                          css.insertRule, self.r1s, 0)
        self.assertRaises(xml.dom.NoModificationAllowedErr,
                          css.deleteRule, 0)

    def test_add_get(self):
        css = cssutils.css.CSSStyleSheet()
        css.addRule(self.r1)
        css.addRule(self.r1)
        css.addRule(self.r1)
        
        self.assertEqual(3, len(css.cssRules)) # with comment


    def test_add_insert(self):
        css1 = cssutils.css.CSSStyleSheet()
        css2 = cssutils.css.CSSStyleSheet()

        css1.addRule(self.r1)
        css1.insertRule(self.r1s, 0)
        css2.insertRule(self.r1, 0)

        self.assertEqual(self.r1.cssText, self.r1s)        

        r1 = css1.cssRules.item(0)        
        r2 = css2.cssRules.item(0)

        self.assertEqual(r1.cssText, r2.cssText)        

    def test_insert(self):
        css = cssutils.css.CSSStyleSheet()

        self.assertRaises(xml.dom.InvalidModificationErr,
                          css.insertRule, u' { }', 0)
        self.assertRaises(xml.dom.InvalidModificationErr,
                          css.insertRule, u'   { color: red;', 0)

        self.assertRaises(xml.dom.SyntaxErr,
                          css.insertRule, u'', 0)        
        self.assertRaises(xml.dom.InvalidModificationErr,
                          css.insertRule, u'{ color: blue;}', 0)
        
        self.assertRaises(xml.dom.InvalidModificationErr,
                          css.insertRule, u'a { color: red;', 0)

        self.assertRaises(xml.dom.SyntaxErr,
                          css.insertRule, u'a { color red; }', 0)
        # only 1 rule allowed
        self.assertRaises(xml.dom.SyntaxErr,
                          css.insertRule, u'a { } b {}', 0)
        self.assertRaises(xml.dom.SyntaxErr,
                          css.insertRule, u'@charset "ascii"; a { }', 0)

        self.assertEqual(0, css.insertRule(u'a { color: red; }', 0))

        self.assertRaises(xml.dom.IndexSizeErr, css.insertRule, self.r1s, 2)

        self.assertEqual(0, css.insertRule(self.r1s, -1))
        self.assertEqual(0, css.insertRule(self.r1s, 0))
        self.assertEqual(2, css.insertRule(self.r1s, 2))
        self.assertEqual(4, len(css.cssRules))

        # position ok?
        css.insertRule('.r2 { color: red; }', 1)
        self.assertEqual('.r2', css.cssRules[1].cssText[:3])        

    def test_insertImport(self):
        css = cssutils.css.CSSStyleSheet()
        css.insertRule('.r2 { color: red; }', 0)
        self.assertRaises(xml.dom.HierarchyRequestErr,
                          css.insertRule, '@import all url(test.css);', 1)
        self.assertEqual(1, len(css.cssRules))
        css.insertRule('@import url(test.css) all;', 0)
        self.assertEqual(2, len(css.cssRules))

        css = cssutils.css.CSSStyleSheet()
        css.insertRule('@import url(test.css) all;', 0)
        css.insertRule('@import url(test.css) print;', 1)
        self.assertEqual(2, len(css.cssRules))


    def test_delete(self):
        css = cssutils.css.CSSStyleSheet()

        css.insertRule(self.r1s, 0)
        css.insertRule(self.r1s, 0)
        css.insertRule(self.r1s, 0)
        css.insertRule(u'#r2 { color: red; }', 1)
        self.assertEqual(4, len(css.cssRules))

        self.assertRaises(xml.dom.IndexSizeErr, css.deleteRule, 4)

        # deleted the right one?
        self.assertEqual(u'#r2', css.cssRules[1].cssText[:3])        
        css.deleteRule(1)
        self.assertEqual(3, len(css.cssRules))
        self.assertEqual(u'.r1', css.cssRules[1].cssText[:3])        

    def test_insertorder(self):
        css = cssutils.css.CSSStyleSheet()

        r0s = u'/* x */'
        r2s = u'.r1 { color: red; }'

        css.insertRule(self.r1s, 0)
        css.insertRule(r2s,0)
        css.insertRule(self.r1s,0)      

    def test_cssText(self):
        text = u"""a {
    color: red;
    }"""        
        css = cssutils.css.CSSStyleSheet()
        css.cssText = text
        self.assertEqual(text, css.cssText)

        # test error
        text = u"""@media unknown"""        
        #self.assertRaises(xml.dom.DOMException, css._setCssText, text)


    def test_selectors(self):
        c = cssutils.css.CSSStyleSheet()
        c.cssText = u'*  {}'
        self.assertEqual(u'* {\n\n    }', c.cssText)
        c.cssText = u'#a  {}'
        self.assertEqual(u'#a {\n\n    }', c.cssText)
        c.cssText = u'e#a  {}'
        self.assertEqual(u'e#a {\n\n    }', c.cssText)
        c.cssText = u'.a  {}'
        self.assertEqual(u'.a {\n\n    }', c.cssText)
        c.cssText = u'e.a  {}'
        self.assertEqual(u'e.a {\n\n    }', c.cssText)

        c.cssText = u'a:hover {}'
        self.assertEqual(u'a:hover {\n\n    }', c.cssText)

        c.cssText = u'a b  {}'
        self.assertEqual(u'a b {\n\n    }', c.cssText)
        c.cssText = u'a *  {}'
        self.assertEqual(u'a * {\n\n    }', c.cssText)
        c.cssText = u'a>b  {}'
        self.assertEqual(u'a>b {\n\n    }', c.cssText)
        c.cssText = u'a+b  {}'
        self.assertEqual(u'a+b {\n\n    }', c.cssText)
        c.cssText = u'a[x] {}'
        self.assertEqual(u'a[x] {\n\n    }', c.cssText)
        c.cssText = u'a[x=1] {}'
        self.assertEqual(u'a[x=1] {\n\n    }', c.cssText)
        c.cssText = u'a[x~=y] {}'
        self.assertEqual(u'a[x~=y] {\n\n    }', c.cssText)
        c.cssText = u'a[x|=y] {}'
        self.assertEqual(u'a[x|=y] {\n\n    }', c.cssText)

        c.cssText = u':lang(de) {}'
        self.assertEqual(u':lang(de) {\n\n    }', c.cssText)
        c.cssText = u'a:lang(de) {}'
        self.assertEqual(u'a:lang(de) {\n\n    }', c.cssText)


if __name__ == '__main__':
    unittest.main()
