"""
tests for encutils.py
"""

__version__ = '0.4'


import httplib
import logging
from StringIO import StringIO as SIO
import sys
import unittest

try:
    import encutils
except ImportError:
    import cssutils.encutils as encutils


# helper log
log = logging.getLogger('encutils')
hdlr = logging.StreamHandler(SIO())      
formatter = logging.Formatter('%(levelname)s\t%(message)s')
hdlr.setFormatter(formatter)
log.addHandler(hdlr)
log.setLevel(logging.INFO)    


class AutoEncodingTestCase(unittest.TestCase):

    def test_splitContentTypeValue(self):
        tests = {
            '  any text  ': ('any text', None),

            'text/html': ('text/html', None),
            'text/xml;charset=utf-8': ('text/xml', 'UTF-8'),
            'text/xml ;charset=UTF-8': ('text/xml', 'UTF-8'),
            ' text/html ; charset = ascii': ('text/html', 'ASCII'),
            ' text/html ; charset = ASCII': ('text/html', 'ASCII'),
            '''
            text/html
            ;
            charset
                =
                ascii               ''': ('text/html', 'ASCII')
        }
        for test, exp in tests.items():
            self.assertEqual(
                exp, encutils._splitContentTypeValue(test, log=log))


    def test_getXMLType(self):
        tests = {
            encutils._APPLICATION_TYPE: 'application/xml',
            encutils._APPLICATION_TYPE: 'application/xml-dtd',
            encutils._APPLICATION_TYPE: 'application/xml-external-parsed-entity',
            encutils._APPLICATION_TYPE: 'application/xhtml+xml',
            encutils._TEXT_TYPE: 'text/xml',
            encutils._TEXT_TYPE: 'text/xml-external-parsed-entity',
            encutils._TEXT_TYPE: 'text/xhtml+xml',
            encutils._NON_XML_TYPE: 'text/html',
            encutils._NON_XML_TYPE: 'text/css',
            encutils._NON_XML_TYPE: 'x/x'
            }
        for exp, test in tests.items():
            self.assertEqual(
                exp, encutils._getXMLType(test, log=log))
        


    def test_encodingByMediaType(self):
        tests = {
            'text/plain': 'ASCII',
            'ANYTHING': 'ASCII',
            'application/xml': 'UTF-8',
            'application/xml-dtd': 'UTF-8',
            'application/xml-external-parsed-entity': 'UTF-8',
            'application/ANYTHING+xml': 'UTF-8',
            '  application/xml  ': 'UTF-8',
            'text/xml': 'ASCII',
            'text/xml-external-parsed-entity': 'ASCII',
            'text/ANYTHING+xml': 'ASCII'}
        for test, exp in tests.items():
            self.assertEqual(exp,
                             encutils.encodingByMediaType(test, log=log))


    def test_getHTTPInfo(self):
        # see test_guessEncoding
        pass
##        tests = {
##            # no default
##            'Content-Type: text/plain': ('text/plain', None),
##            'Content-Type: text/html': ('text/html', None),
##            # default
##            'Content-Type: text/xml': ('text/xml', 'ascii'),
##            'Content-Type: application/xml': ('application/xml', 'utf-8'),
##            # explicit charset
##            'Content-Type: text/html; charset=iso' : ('text/html', 'iso'),
##            'Content-Type: application/xml ; charset= iso': (
##                'application/xml', 'iso'),
##            # no content-type
##            'Content-x: application/xml': (None, None)
##            }
##        for test, exp in tests.items():
##            k, v = test.split(':')
##            test = {k: v}
##            self.assertEqual(exp, encutils.getHTTPInfo(test, log=log))
            
    
    def test_getMetaInfo(self):
        tests = {
            """<meta ttp-equiv='Content-Type' content='text/html; charset=ascii'>""":
                (None, None),
            """<meta http-equiv='ontent-Type' content='text/html; charset=ascii'>""":
                (None, None),
            """<meta http-equiv='Content-Type' content='text/html'>""":
                ('text/html', None),
            """<meta http-equiv='Content-Type' content='text/html ;charset=ascii'>""":
                ('text/html', 'ASCII'),
            """<meta content='text/html;charset=iso-8859-1' http-equiv='Content-Type'>""":
                ('text/html', 'ISO-8859-1'),
            """<meta http-equiv="Content-Type" content="text/html;charset = ascii">""":
                ('text/html', 'ASCII'),
            """<meta http-equiv='Content-Type' content="text/html;charset=ascii">""":
                ('text/html', 'ASCII'),
            """<meta http-equiv='Content-Type' content='text/html;charset=ascii'  />""":
                ('text/html', 'ASCII'),
            """<meta http-equiv = " Content-Type" content = " text/html;charset=ascii " >""":
                ('text/html', 'ASCII'),
            """<meta http-equiv = " \n Content-Type " content = "  \t text/html   ;  charset=ascii " >""":
                ('text/html', 'ASCII')
            }
        for test, exp in tests.items():
            self.assertEqual(exp, encutils.getMetaInfo(test, log=log))


    def test_getXMLEncoding(self):
        tests = {
            ('UTF-8', False): '<?xml version="1.0" ?>', 
            ('ASCII', True): '<?xml version="1.0" encoding="ascii" ?>', 
            ('ASCII', True): "<?xml version='1.0' encoding='ascii' ?>", 
            ('ISO-8859-1', True): "<?xml version='1.0' encoding='iso-8859-1' ?>", 
            ('UTF-8', False): '<?xml version="1.0" ?><x encoding="ascii"/>'
            }
        for exp, test in tests.items():
            self.assertEqual(exp, encutils.getXMLEncoding(test, log=log))        


    def test_guessEncoding(self):

        def _fakeRes(content):
            "build a fake HTTP response"

            class FakeRes:

                def __init__(self, content):
                    fp = SIO(content)
                    self._info = httplib.HTTPMessage(fp)
                    
                def info(self):
                    return self._info

            return FakeRes(content)

        # (expectedencoding, expectedmismatch): (httpheader, filecontent)
        tests = {
            
            # --- application/xhtml+xml ---
            # default enc
            ('UTF-8', False): (
                '''Content-Type: application/xhtml+xml''',
                '''<?xml version="1.0" ?>
                    <example>
                        <meta http-equiv="Content-Type"
                            content="application/xhtml+xml"/>
                    </example>'''),
            # header enc
            ('ISO-H', False): (
                '''Content-Type: application/xhtml+xml;charset=iso-H''',
                '''<?xml version="1.0" ?>
                    <example>
                        <meta http-equiv="Content-Type"
                            content="application/xhtml+xml"/>
                    </example>'''),
            # mismatch header - meta, meta ignored
            ('ISO-H', True): (
                '''Content-Type: application/xhtml+xml;charset=iso-H''',
                '''<?xml version="1.0" ?>
                    <example>
                        <meta http-equiv="Content-Type"
                            content="application/xhtml+xml;charset=iso_M"/>
                    </example>'''),
            # mismatch XML - meta, meta ignored
            ('ISO-X', True): (
                '''Content-Type: application/xhtml+xml''',
                '''<?xml version="1.0" encoding="iso-X" ?>
                    <example>
                        <meta http-equiv="Content-Type"
                            content="application/xhtml+xml;charset=iso_M"/>
                    </example>'''),
            # mismatch header and XML, header wins
            ('ISO-H', True): (
                '''Content-Type: application/xhtml+xml;charset=iso-H''',
                '''<?xml version="1.0" encoding="iso-X" ?>
                    <example/>'''),

            # --- text/xml ---
            # default enc
            ('ASCII', False): (
                '''Content-Type: text/xml''',
                '''<?xml version="1.0" ?>
                    <example>
                        <meta http-equiv="Content-Type"
                            content="text/xml"/>
                    </example>'''),
            # header enc
            ('ISO-H', False): (
                '''Content-Type: text/xml;charset=iso-H''',
                '''<?xml version="1.0" ?>
                    <example>
                        <meta http-equiv="Content-Type"
                            content="text/xml"/>
                    </example>'''),
            # mismatch header - meta, meta ignored
            ('ISO-H', True): (
                '''Content-Type: text/xml;charset=iso-H''',
                '''<?xml version="1.0" ?>
                    <example>
                        <meta http-equiv="Content-Type"
                            content="text/xml;charset=iso_M"/>
                    </example>'''),
            # mismatch XML - meta, both ignored!
            ('ASCII', True): (
                '''Content-Type: text/xml''',
                '''<?xml version="1.0" encoding="iso-X" ?>
                    <example>
                        <meta http-equiv="Content-Type"
                            content="text/xml;charset=iso_M"/>
                    </example>'''),
            # mismatch header and XML, XML ignored
            ('ISO-H', True): (
                '''Content-Type: text/xml;charset=iso-H''',
                '''<?xml version="1.0" encoding="iso-X" ?>
                    <example/>'''),

            # --- text/html ---
            # default enc
            ('ASCII', False): ('Content-Type: text/html;',
                '''<meta http-equiv="Content-Type"
                    content="text/html">'''),
            # header enc
            ('ISO-H', False): ('Content-Type: text/html;charset=iso-H',
                '''<meta http-equiv="Content-Type"
                    content="text/html">'''),
            # meta enc
            ('ISO-M', False): ('Content-Type: text/html',
                '''<meta http-equiv="Content-Type"
                    content="text/html;charset=iso-m">'''),
            # mismatch header - meta, header wins
            ('ISO-H', True): ('Content-Type: text/html;charset=iso-H',
                '''<meta http-equiv="Content-Type"
                    content="text/html;charset=iso-m">''')           
            }
        for exp, test in tests.items():
            header, content = test
            self.assertEqual(exp, encutils.guessEncoding(
                _fakeRes(header), content, log=log))


if __name__ == '__main__':
    unittest.main()
